%   Implements a Metadata structure
%
%   myMetaData = SP.metadata
%
%--------------------------------------------------------------------------
% Copyright (c) 2014, Florian Heese
% Institute of Communication Systems and Data Processing
% RWTH Aachen University, Germany
% Contact information: heese@ind.rwth-aachen.de
%
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the 
%      distribution
%    * Neither the name of the RWTH Aachen University nor the names
%      of its contributors may be used to endorse or promote products 
%      derived from this software without specific prior written 
%      permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
% PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
% OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%--------------------------------------------------------------------------
classdef metadata < dynamicprops
   
   properties
      date
      description
      notes
   end
   
   methods
      function this = metadata()
         this.date = now;
      end
      
      % shows the fields and values
      function display(this)
         disp(strcat(inputname(1), ' ='))
         my_props = properties(this);
         for i = 1:size(my_props, 1);
            fprintf('%20s: ', my_props{i})
            if strcmp(my_props{i},'date')
               fprintf('%s\n', datestr(this.(my_props{i})));
            elseif ischar(this.(my_props{i}))
               fprintf('%s\n', this.(my_props{i}));
            elseif isnumeric(this.(my_props{i}))
               fprintf('%d\n', this.(my_props{i}));
            else
               fprintf('{...}\n');
            end
         end
      end
      
      % adds a field with name propertyName and the related data to the
      % metadata
      function addMetaData(this, propertyName, data)
         p = this.addprop(propertyName);
         this.(propertyName) = data;
         % p.Transient = true;
      end
      
      % returns a cell array containing all objects of the class className
      function results = getMetaDataByClass(this, className)
         results = {};
         my_probs = properties(this);
         for i=1:size(my_probs,1)
             if isa(this.(my_probs{i}), className)
                 results{end+1} = this.(my_probs{i});
             end
         end
      end
      
      % checks if there is a field with named fieldname
      function bool = isfield(this, fieldname)                 
         bool = false;
         if ~ischar(fieldname)
             error('The input has to be a string!');
         end
         my_probs = properties(this);
         for i=1:size(my_probs,1)
             if strcmpi(fieldname,my_probs{i})
                 bool = true;
                 break;
             end
         end
      end

      % Since SP.metadata is a handle object using the = Operator copies
      % only its reference. Therefor a specific copy method is necessary.
      function copy = copy(this)
         copy = SP.metadata();
         my_props = properties(this);
         for i = 1:size(my_props, 1);
             my_prop = my_props{i};
             if isobject(my_prop)
                 the_copy = copy(this.(my_prop));
             else
                 the_copy = this.(my_prop);
             end
             if isprop(copy, my_prop)
                copy.(my_prop) = the_copy;
             else
                copy.addMetaData(my_prop, the_copy);
             end                 
         end
         
      end      
      
   end
   
end

