% conv_it(conv_signal) convolves current signal with conv_signal and
%    replaces current signal.
%
%    signal1.conv_it(signal2);
%
%--------------------------------------------------------------------------
% Copyright (c) 2014, Florian Heese
% Institute of Communication Systems and Data Processing
% RWTH Aachen University, Germany
% Contact information: heese@ind.rwth-aachen.de
%
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the 
%      distribution
%    * Neither the name of the RWTH Aachen University nor the names
%      of its contributors may be used to endorse or promote products 
%      derived from this software without specific prior written 
%      permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
% PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
% OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%--------------------------------------------------------------------------
function conv_it(this, b)

   if ~isa(b, 'SP.signal')
       if isnumeric(b)
           warning('Assuming %s to be at fs=%i.', inputname(1), this.fs)
           b = SP.signal(b, this.fs);
       else
           error('Argument is not a SP.signal or a vector or a matric!')
       end
   end

   % check for channel missmatch
   channels = max(this.channels, b.channels);
   if (this.channels > b.channels) && (b.channels == 1)
       ch_lookup_this = 1:this.channels;
       ch_lookup_b = ones(channels,1);
   elseif (this.channels < b.channels) && (this.channels == 1)
       ch_lookup_b = 1:b.channels;
       ch_lookup_this = ones(channels,1);
   else
       error('Could not resolv channels missmatch!')
   end

   % Convolving...
   tmp_data = zeros(this.signalSamples + b.signalSamples -1, channels);
   for c=1:channels
    t_idx = ch_lookup_this(c);
    b_idx = ch_lookup_b(c);
    tmp_data(':',c) = conv(this.to_double(':',t_idx), b.to_double(':',b_idx));
   end
   
   this.setSignal(tmp_data);
end

