% horzcat(varargin) concatenates an arbitrary number N of signals. The 
%   result is signal containing the input signals in the committed order,
%
%   newSignal = horzcat(signal_1, signal_2, ..., signal_N)
%   newSignal = [ signal_1, signal_2, ..., signal_N ]
%
%--------------------------------------------------------------------------
% Copyright (c) 2014, Florian Heese
% Institute of Communication Systems and Data Processing
% RWTH Aachen University, Germany
% Contact information: heese@ind.rwth-aachen.de
%
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the 
%      distribution
%    * Neither the name of the RWTH Aachen University nor the names
%      of its contributors may be used to endorse or promote products 
%      derived from this software without specific prior written 
%      permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
% PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
% OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%--------------------------------------------------------------------------
function [ signal ] = horzcat(varargin)

   % safety first
   if nargin == 1
      signal = varargin{1};
      return
   end

   common_channels = varargin{1}.channels;
   for idx = 2:nargin
      if (common_channels ~= varargin{idx}.channels)
         error('Channels count of %s and %s do not match!), this.sourceName, a.sourceName')
      end
   end
   
   % find common dst_fs
   dst_fs = varargin{1}.fs;
   for idx = 2:nargin
      if varargin{idx}.fs > dst_fs
         warning('fs of %s does not match the previius! Resampling to dst_fs Hz', varargin{idx}.sourceName, dst_fs)
         dst_fs = varargin{idx}.fs;
      end
   end

   % compose the concatenated signal
   tmp_signal = [];
   for idx=1:nargin
      % resample if necessary
      if (varargin{idx}.fs ~= dst_fs)
        tmp = resample(varargin{idx}.to_double, dst_fs, varargin{idx}.fs);
      else
        tmp = varargin{idx}.to_double;
      end
      tmp_signal = [tmp_signal; tmp];
      dst_SourceName{idx} = varargin{idx}.sourceName;
   end

   signal = SP.signal(tmp_signal, dst_fs);
   signal.sourceType = ['from horzcat ' num2str(nargin) ' SP.signal(s)'];
   signal.sourceName = SP.strjoin(', ', dst_SourceName);
   SP.joinMetadata(signal, varargin{:});
   
end

