% times(factor) multiplies the samples of SP.signal object by factor.
%
%   newSignal = times(signal, factor) 
%   newSignal = signal .* factor
%   where signal is a SP.signal object and factor is a scalar.
%
%--------------------------------------------------------------------------
% Copyright (c) 2014, Florian Heese
% Institute of Communication Systems and Data Processing
% RWTH Aachen University, Germany
% Contact information: heese@ind.rwth-aachen.de
%
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the 
%      distribution
%    * Neither the name of the RWTH Aachen University nor the names
%      of its contributors may be used to endorse or promote products 
%      derived from this software without specific prior written 
%      permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
% PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
% OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%--------------------------------------------------------------------------
function [ signal ] = times(a, factor )

   if ~isa(a, 'SP.signal')
      tmp = factor;
      factor = a;
      a = tmp;
   end
   
   % in the SP.signal constructor there is a differentation whether a 
   % numeric input parameter is greater than 2, so if you apply the factor
   % to the double_signal and it gets greater than 2 the new data will just
   % be stored in the short property, which is not useful.
   if max(abs(a.to_double .*factor)) > 2                                    
       warning('Signal amplitude > 2. Signal will be cropped!');
       vec = a.to_double .* factor;
       vec(abs(vec)>2) = 2;
       signal = SP.signal(vec, a.fs);
   else
       signal = SP.signal(a.to_double .* factor, a.fs);
   end
   signal.sourceType = a.sourceType;

   signal.sourceName = [ 'scaled (factor=' num2str(factor) ') version of ' a.sourceName ];
   SP.joinMetadata(signal, a, a);

end

