% truncates each passed SP.signal to the length of the shortest passed.
%
%--------------------------------------------------------------------------
% Copyright (c) 2014, Florian Heese
% Institute of Communication Systems and Data Processing
% RWTH Aachen University, Germany
% Contact information: heese@ind.rwth-aachen.de
%
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the 
%      distribution
%    * Neither the name of the RWTH Aachen University nor the names
%      of its contributors may be used to endorse or promote products 
%      derived from this software without specific prior written 
%      permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
% PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
% OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%--------------------------------------------------------------------------
function [varargout] = truncate(varargin)

   % Safety Checks
   if nargin < 2
      error('You need at least 2 signals...')
   end

   common_fs = varargin{1}.fs;

   for idx=2:nargin
      if varargin{idx}.fs ~= common_fs
         error('Samplefrequency of input signals does not match!')
      end
   end
   
   process = 0;
   
   common_sample_amount = varargin{1}.signalSamples;

   % find shortest SP.signal
   for idx=2:nargin
      if varargin{idx}.signalSamples < common_sample_amount                % if current signal is shorter than all previous signals, truncate
         common_sample_amount = varargin{idx}.signalSamples;
         process = 1; 
      elseif varargin{idx}.signalSamples > common_sample_amount;           % if regarded signal is longer than 1st signal, truncate 
         process = 1; 
      end
   end

   if (process == 1)
       truncated_signals_names = [];
       for idx=1:nargin
           if ~(varargin{idx}.signalSamples == common_sample_amount)
               x = varargin{idx}.to_double(1:common_sample_amount ,':');
               varargin{idx}.setSignal(x);
               truncated_signals_names{end+1} = varargin{idx}.sourceName;
           end
           theoutputs{idx} = varargin{idx};
       end
       varargout = theoutputs(1:nargout);
       trunc_signals_text = sprintf('* %s\n', truncated_signals_names{:});
       warning('SP.signal(s): \n%sare truncated to %.2f[s]!', ...
               trunc_signals_text, ...
               common_sample_amount / common_fs);
   else
       varargout = varargin;
   end

end

