%--------------------------------------------------------------------------
% Copyright (c) 2014, Florian Heese
% Institute of Communication Systems and Data Processing
% RWTH Aachen University, Germany
% Contact information: heese@ind.rwth-aachen.de
%
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the 
%      distribution
%    * Neither the name of the RWTH Aachen University nor the names
%      of its contributors may be used to endorse or promote products 
%      derived from this software without specific prior written 
%      permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
% PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
% OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%--------------------------------------------------------------------------

% Available Microphone Setups
% -------------------------------------------------------------------------
%   * Spacing 1 (3-3-3-8-3-3-3) cm
%   * Spacing 2 (4-4-4-8-4-4-4) cm
%   * Spacing 3 (8-8-8-8-8-8-8) cm
% -------------------------------------------------------------------------

% Available Reverberation Times
% -------------------------------------------------------------------------
%   * 0.160 [s]
%   * 0.360 [s]
%   * 0.610 [s]
% -------------------------------------------------------------------------

% Room - Setup
% -------------------------------------------------------------------------
%
%          +-----------------window-----------------+
%          |                                        |
%          |90°  -1-2-3-4- mic array -5-6-7-8-  270°|
%          |                                        |
%          | 75°                               285° |
%          |                                        |
%          |  60°                             300°  |
%          |                                        |
%          |    45°                         315°    |
%          |                                        |
%          |       30°                   330°       |     
%          |             15°        345°            |         
%          |                   0°                   |
%          |                                        |
%          +------------------door------------------+
%
% Angle:    180° in 15° steps
% distance: 1m, 2m (from mic array center)
% -------------------------------------------------------------------------
% Angle mapping according the notation used within the CoVR project 
% (http://www.covr.rwth-aachen.de):
% 0 frontal, 90 left, 180 back, 270 right
% -------------------------------------------------------------------------

fs = 48000;
reverberation = 0.160;
mic_spacing = '3-3-3-8-3-3-3';

% YOU HAVE TO ADAPT DB_PATH
db_base_path = '/share/fhData/CoVR_Recordings/2014_CoVR_Database_publication/Impulse_Responses';
% YOU HAVE TO ADAPT DB_PATH

% create 10s clean white noise signal
clean_noise = SP.signal('WhiteNoise', 10, fs, 1);

% load "dry" clean speech
clean_speech_1 = SP.signal(fullfile('clean_speech_samples', 'a0336.wav'));
clean_speech_2 = SP.signal(fullfile('clean_speech_samples', 'a0345.wav'));

% convolve clean_speech_1 with Room Impulse Response
speech_1 = 0.5 .* clean_speech_1.conv(SP.loadImpulseResponse(...
    db_base_path, ...
    mic_spacing, ...
    0, ...                 % angle
    1, ...                 % distance
    reverberation, ...
    0.5));                 % truncate impulse response to 0.5 [s]

% convolve clean_speech_2 with Room Impulse Response
speech_2 = 0.5 .* clean_speech_2.conv(SP.loadImpulseResponse(...
    db_base_path, ...
    mic_spacing, ...
    90, ...                % angle
    2, ...                 % distance
    reverberation, ...
    0.5));                 % truncate impulse response to 0.5 [s]

% convolve clean_noise with Room Impulse Response
noise = clean_noise.conv(SP.loadImpulseResponse(...
    db_base_path, ...
    mic_spacing, ...
    315, ...               % angle
    1, ...                 % distance
    reverberation, ...
    0.5));                 % truncate impulse response to 0.5 [s]

% truncate all convolved signals to a common length
SP.truncate(speech_1, speech_2, noise);

% calculate scaling factors for desired SNRs
[ ~, ~, ~, fac_noise ]    = SP.adjustSNR(speech_1, noise, 10, 4, 4);
[ ~, ~, ~, fac_speech_2 ] = SP.adjustSNR(speech_1, speech_2, 0, 4, 4);

% create final sound field by superposition
sound_field = speech_1 + speech_2 .* fac_speech_2 + noise .* fac_noise;

sound_field.write_wav('sample_soundfield_output.wav')

sound_field_stereo_downmix = SP.signal(...
    sound_field.to_double(':', [1 8]), ...
    sound_field.fs);

sound_field_stereo_downmix.write_wav('sample_soundfield_output_stereo.wav')

my_samples = sound_field.to_double; % dim: (samples, channels)
